// ignore_for_file: non_constant_identifier_names, camel_case_types, missing_return, unused_import, unused_local_variable, dead_code, unnecessary_cast
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tencent_live_fluttify/src/android/android.export.g.dart';
import 'package:tencent_live_fluttify/src/ios/ios.export.g.dart';

import 'enums.dart';

class LivePlayer {
  LivePlayer._();

  /// 创建一个播放器
  ///
  /// 直播和点播
  /// - 直播（LIVE）的视频源是主播实时推送的。因此，主播停止推送后，播放端的画面也会随即停止，而且由于是实时直播，所以播放器在播直播 URL 的时候是没有进度条的。
  /// - 点播（VOD）的视频源是云端的一个视频文件，只要未被从云端移除，视频就可以随时播放， 播放中您可以通过进度条控制播放位置，腾讯视频和优酷、土豆等视频网站上的视频观看就是典型的点播场景。
  ///
  /// 协议的支持
  /// - FLV 优点: 成熟度高、高并发无压力 缺点: 需集成 SDK 才能播放 播放延迟: 2s - 3s
  /// - RTMP 优点: 优质线路下理论延迟最低	缺点: 高并发情况下表现不佳	播放延迟: 1s - 3s
  /// - HLS（m3u8） 优点: 手机浏览器支持度高 缺点: 延迟非常高	播放延迟: 10s - 30s
  static Future<LivePlayer> create() async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  com_tencent_rtmp_TXLivePlayer _androidPlayer;
  TXLivePlayer _iosPlayer;

  com_tencent_rtmp_TXLivePlayConfig _androidConfig;
  TXLivePlayConfig _iOSConfig;

  final _androidEventDelegate = _AndroidEventDelegate();
  final _iosEventDelegate = _IOSEventDelegate();

  /// 设置播放界面
  Future<void> setPlayerView(dynamic playerView) async {
    assert(
      playerView is com_tencent_rtmp_ui_TXCloudVideoView ||
          playerView is UIView,
    );
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 开始播放
  Future<void> startPlay(
    String playUrl, {
    PlayType type = PlayType.RTMP,
  }) async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 恢复播放
  Future<void> resumePlay() async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 恢复直播
  Future<int> resumeLive() async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 是否在播放中
  Future<bool> isPlaying() async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 暂停播放
  Future<void> pausePlay() async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 停止播放
  Future<void> stopPlay() async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 清晰度无缝切换
  ///
  /// 日常使用中，网络情况在不断发生变化。在网络较差的情况下，最好适度降低画质，以减少卡顿；反之，网速比较好，可以提高观看画质。
  /// 传统切流方式一般是重新播放，会导致切换前后画面衔接不上、黑屏、卡顿等问题。使用无缝切换方案，在不中断直播的情况下，能直接切到另条流上。
  /// 清晰度切换在直播开始后，任意时间都可以调用。
  Future<void> switchStream(String newStream) async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 画面调整
  ///
  /// 铺满or适应
  Future<void> setRenderMode(RenderMode renderMode) async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }

  /// 事件处理
  Future<void> setOnEventListener({
    VoidCallback onWarningVideoDecodeFail,
    VoidCallback onWarningAudioDecodeFail,
    VoidCallback onWarningReconnect,
    VoidCallback onWarningRecvDataLag,
    VoidCallback onWarningVideoPlayLag,
    VoidCallback onWarningHwAccelerationFail,
    VoidCallback onWarningVideoDiscontinuity,
    VoidCallback onWarningDNSFail,
    VoidCallback onWarningServerConnFail,
    VoidCallback onWarningShakeFail,
    VoidCallback onEventRcvFirstIFrame,
    VoidCallback onEventPlayBegin,
  }) async {
    debugPrint('此为付费插件, 请联系qq 382146139购买');
  }
}

class _AndroidEventDelegate extends java_lang_Object
    with com_tencent_rtmp_ITXLivePlayListener {
  VoidCallback _onWarningVideoDecodeFail;
  VoidCallback _onWarningAudioDecodeFail;
  VoidCallback _onWarningReconnect;
  VoidCallback _onWarningRecvDataLag;
  VoidCallback _onWarningVideoPlayLag;
  VoidCallback _onWarningHwAccelerationFail;
  VoidCallback _onWarningVideoDiscontinuity;
  VoidCallback _onWarningDNSFail;
  VoidCallback _onWarningServerConnFail;
  VoidCallback _onWarningShakeFail;

  VoidCallback _onEventRcvFirstIFrame;
  VoidCallback _onEventPlayBegin;

  @override
  Future<void> onPlayEvent(int var1, android_os_Bundle var2) async {
    super.onPlayEvent(var1, var2);
    debugPrint('onPlayEvent: $var1');
    // 当前视频帧解码失败
    if (var1 == 2101 && _onWarningVideoDecodeFail != null) {
      _onWarningVideoDecodeFail();
    }
    // 当前音频帧解码失败
    else if (var1 == 2102 && _onWarningAudioDecodeFail != null) {
      _onWarningAudioDecodeFail();
    }
    // 网络断连，已启动自动重连（重连超过三次就直接抛送 PLAY_ERR_NET_DISCONNECT）
    else if (var1 == 2103 && _onWarningReconnect != null) {
      _onWarningReconnect();
    }
    // 网络来包不稳：可能是下行带宽不足，或由于主播端出流不均匀
    else if (var1 == 2104 && _onWarningRecvDataLag != null) {
      _onWarningRecvDataLag();
    }
    // 当前视频播放出现卡顿
    else if (var1 == 2105 && _onWarningVideoPlayLag != null) {
      _onWarningVideoPlayLag();
    }
    // 硬解启动失败，采用软解
    else if (var1 == 2106 && _onWarningHwAccelerationFail != null) {
      _onWarningHwAccelerationFail();
    }
    // 当前视频帧不连续，可能丢帧
    else if (var1 == 2107 && _onWarningVideoDiscontinuity != null) {
      _onWarningVideoDiscontinuity();
    }
    // RTMP - DNS 解析失败（会触发重试流程） 3001
    else if (var1 == 3001 && _onWarningDNSFail != null) {
      _onWarningDNSFail();
    }
    // RTMP 服务器连接失败（会触发重试流程） 3002
    else if (var1 == 3002 && _onWarningServerConnFail != null) {
      _onWarningServerConnFail();
    }
    // RTMP 服务器握手失败（会触发重试流程） 3003
    else if (var1 == 3003 && _onWarningShakeFail != null) {
      _onWarningShakeFail();
    }
    // 收到首帧数据，越快收到此消息说明链路质量越好
    else if (var1 == 2003 && _onEventRcvFirstIFrame != null) {
      _onEventRcvFirstIFrame();
    }
    // 视频播放开始，如果您自己做 loading，会需要它
    else if (var1 == 2004 && _onEventPlayBegin != null) {
      _onEventPlayBegin();
    }
  }

  @override
  Future<void> onNetStatus(android_os_Bundle var1) async {
    super.onNetStatus(var1);
  }
}

class _IOSEventDelegate extends NSObject with TXLivePlayListener {
  VoidCallback _onWarningVideoDecodeFail;
  VoidCallback _onWarningAudioDecodeFail;
  VoidCallback _onWarningReconnect;
  VoidCallback _onWarningRecvDataLag;
  VoidCallback _onWarningVideoPlayLag;
  VoidCallback _onWarningHwAccelerationFail;
  VoidCallback _onWarningVideoDiscontinuity;
  VoidCallback _onWarningDNSFail;
  VoidCallback _onWarningServerConnFail;
  VoidCallback _onWarningShakeFail;

  VoidCallback _onEventRcvFirstIFrame;
  VoidCallback _onEventPlayBegin;

  @override
  Future<void> onPlayEvent_withParam(int EvtID, Map param) async {
    super.onPlayEvent_withParam(EvtID, param);
    debugPrint('onPlayEvent_withParam: $EvtID, $param');
    // 当前视频帧解码失败
    if (EvtID == 2101 && _onWarningVideoDecodeFail != null) {
      _onWarningVideoDecodeFail();
    }
    // 当前音频帧解码失败
    else if (EvtID == 2102 && _onWarningAudioDecodeFail != null) {
      _onWarningAudioDecodeFail();
    }
    // 网络断连，已启动自动重连（重连超过三次就直接抛送 PLAY_ERR_NET_DISCONNECT）
    else if (EvtID == 2103 && _onWarningReconnect != null) {
      _onWarningReconnect();
    }
    // 网络来包不稳：可能是下行带宽不足，或由于主播端出流不均匀
    else if (EvtID == 2104 && _onWarningRecvDataLag != null) {
      _onWarningRecvDataLag();
    }
    // 当前视频播放出现卡顿
    else if (EvtID == 2105 && _onWarningVideoPlayLag != null) {
      _onWarningVideoPlayLag();
    }
    // 硬解启动失败，采用软解
    else if (EvtID == 2106 && _onWarningHwAccelerationFail != null) {
      _onWarningHwAccelerationFail();
    }
    // 当前视频帧不连续，可能丢帧
    else if (EvtID == 2107 && _onWarningVideoDiscontinuity != null) {
      _onWarningVideoDiscontinuity();
    }
    // RTMP - DNS 解析失败（会触发重试流程） 3001
    else if (EvtID == 3001 && _onWarningDNSFail != null) {
      _onWarningDNSFail();
    }
    // RTMP 服务器连接失败（会触发重试流程） 3002
    else if (EvtID == 3002 && _onWarningServerConnFail != null) {
      _onWarningServerConnFail();
    }
    // RTMP 服务器握手失败（会触发重试流程） 3003
    else if (EvtID == 3003 && _onWarningShakeFail != null) {
      _onWarningShakeFail();
    }
    // 收到首帧数据，越快收到此消息说明链路质量越好
    else if (EvtID == 2003 && _onEventRcvFirstIFrame != null) {
      _onEventRcvFirstIFrame();
    }
    // 视频播放开始，如果您自己做 loading，会需要它
    else if (EvtID == 2004 && _onEventPlayBegin != null) {
      _onEventPlayBegin();
    }
  }

  @override
  Future<void> onNetStatus(Map param) async {
    super.onNetStatus(param);
  }
}
